﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;
namespace HeuristicLab.Problems.Instances.Regression {
  public class ValueGenerator {
    protected static FastRandom rand = new FastRandom();

    public static double[,] Transformation(List<List<double>> data) {
      double[,] values = new double[data.First().Count, data.Count];
      for (int i = 0; i < values.GetLength(0); i++) {
        for (int j = 0; j < values.GetLength(1); j++) {
          values[i, j] = data[j][i];
        }
      }
      return values;
    }

    public static List<double> GenerateSteps(double start, double end, double stepWidth) {
      return Enumerable.Range(0, (int)Math.Round(((end - start) / stepWidth) + 1))
                                      .Select(i => (start + i * stepWidth))
                                      .ToList<double>();
    }

    public static List<double> GenerateUniformDistributedValues(int amount, double start, double end) {
      List<double> values = new List<double>();
      for (int i = 0; i < amount; i++) {
        values.Add(rand.NextDouble() * (end - start) + start);
      }
      return values;
    }

    public static List<double> GenerateNormalDistributedValues(int amount, double mu, double sigma) {
      List<double> values = new List<double>();
      for (int i = 0; i < amount; i++) {
        values.Add(NormalDistributedRandom.NextDouble(rand, mu, sigma));
      }
      return values;
    }

    public static List<List<double>> GenerateAllCombinationsOfValuesInLists(List<List<double>> sets) {

      var combinations = new List<List<double>>();

      foreach (var value in sets[0])
        combinations.Add(new List<double> { value });

      foreach (var set in sets.Skip(1))
        combinations = AddListToCombinations(combinations, set);

      combinations = (from i in Enumerable.Range(0, sets.Count)
                      select (from list in combinations
                              select list.ElementAt(i)).ToList<double>()).ToList<List<double>>();

      return combinations;
    }

    private static List<List<double>> AddListToCombinations
         (List<List<double>> combinations, List<double> set) {
      var newCombinations = from value in set
                            from combination in combinations
                            select new List<double>(combination) { value };

      return newCombinations.ToList();
    }
  }
}
